% Compute multiplication matrix Lj
%    P = H-basis
%    LP = Leading terms of P
%    dP = degrees of P
%    N = Homogeneous polynomials
%    dN = degree
%    s = #variables

function R = Reduce( P,LP,dP,N,dN,s )
  fdeg = bincoeff( dN+s,s );
  hdeg = bincoeff( dN+s-1,s-1 );
  [ Nm,Nn ] = size( N );

  % Find indices
  ai = find( dP == dN );
  Pkdim = length( ai );

  if Pkdim == 0
    % nothing to reduce
    R{dN+1} = N;
    for l=0:dN-1
      R{l+1} = zeros( bincoeff( l+s-1,s-1 ),Nn );
    end
    return
  else
    LPk = LP( fdeg-hdeg+1:fdeg,ai );
    [ QPk,RPk ] = qr ( LPk );
    
    X = RPk(1:Pkdim,1:Pkdim) \ ( QPk( :,1:Pkdim )' * N );
    R{dN+1} = N - LPk * X;
    T = -P( 1:fdeg-hdeg,ai ) * X;
  end

  % Reduce the rest

  for k=dN-1:-1:0
    fdeg = bincoeff( k+s,s );
    hdeg = bincoeff( k+s-1,s-1 );

    LT = T( fdeg-hdeg+1:fdeg,: );
    % Find indices
    ai = find( dP == k );
    Pkdim = length( ai );

    if Pkdim == 0
      % nothing to reduce
      R{k+1} = LT;
      T = T( 1:fdeg-hdeg,: );
    else
      LPk = LP( fdeg-hdeg+1:fdeg,ai );
      [ QPk,RPk ] = qr ( LPk );
      
      X = RPk(1:Pkdim,1:Pkdim) \ ( QPk( :,1:Pkdim )' * LT );
      R{k+1} = LT - LPk * X;

      T = T( 1:fdeg-hdeg,: ) - P( 1:fdeg-hdeg,ai ) * X;
  end
end

